//*****************************************************************************
// File Name            : nios2e_sys.v
//-----------------------------------------------------------------------------
// Function             : sd_catd sample application
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2013/02/19
// -.-- 2013/06/12
// -.-- 2014/01/23
//*****************************************************************************
module nios2e_sys   ( clk_50m, resetb, 
                   UART_Tx,UART_Rx,led,
                   sw0,sw1,
                   sclk,miso,mosi,ss,
                   mon

                        
);
  //
  parameter p_wb_offset_low  =  32'h10000000;
  parameter p_wb_offset_hi   =  32'h2FFFFFFF;
  parameter p_wb_pwm0_low   =  p_wb_offset_low + 0;
  parameter p_wb_pwm0_hi    =  p_wb_offset_low + 15;
  parameter p_wb_pwm1_low   =  p_wb_offset_low + 16;
  parameter p_wb_pwm1_hi    =  p_wb_offset_low + 31;
  parameter p_wb_spi_low   =   p_wb_offset_low + 256;
  parameter p_wb_spi_hi   =    p_wb_offset_low + 511;
  parameter p_wb_bram_low =    p_wb_offset_low + 32'h0001000;
  parameter p_wb_bram_hi =     p_wb_offset_low + 32'h00017ff;
  //
  input clk_50m;    //clk 50MHz
  input resetb;     //reset_b
  //UART
  output UART_Tx;       //serial tx data   115200Kbps
  input  UART_Rx;       //serial rx data   115200Kbps
  //LED
  output [9:0] led;
  //SW
  input           sw0;
  input           sw1;
  //SPI
  output         sclk ; //spi clk
  input          miso ; //spi master input data
  output         mosi ; //spi master output data
  output         ss ;   //spi select
  //
  //
  output          mon;
  //
  wire  clk_50m_i;  // 50MHz 
  wire  clk_100m_i; //100MHz CLk
  wire  clk_20m_i;  // 2MHz 
  wire  locked;
  wire  reset_loacked;  
  wire  resetb_loacked;   
  //nios2 gpio
  wire  [31:0]   gpo;
  wire  [31:0]   gpi;
  //block ram port-A
  wire      clka; 
  wire  [3:0]wea;
  wire  we_a; 
  wire  [8 : 0] addra;
  wire  [31 : 0] dina;
  wire  [31 : 0] douta;
  //block ram port-B
  wire      clkb; 
  wire  [3:0]web;
  wire  we_b; 
  wire  [8 : 0] addrb;
  wire  [31 : 0] dinb;
  wire  [31 : 0] doutb;
  //wishboen bus  nios2 -> IP
  wire         WB_CLK;         // clock 
  wire         WB_RST;         // reset (asynchronous active hi)
  wire         WB_CYC;         // cycle
  wire         WB_STB;         // strobe
  wire  [31:0] WB_ADR;         // address adr_i[1]
  wire         WB_WE;          // write enable
  wire  [31:0] WB_DAT_I;          // data output
  wire  [31:0] WB_DAT_O;          // data input
  wire  [3:0]  WB_SEL;         // byte enable
  wire         WB_ACK;         // normal bus termination
  reg          WB_ACK_DFF;         // normal bus termination
  wire         WB_ERR;         // err bus termination
  wire         WB_RTY;         // rtry
  //pwm
  /* 
  wire         cyc_i_pwm;         // cycle
  wire         stb_i_pwm;         // strobe
  wire         we_i_pwm;          // write enable
  wire  [ 7:0] dat_i_pwm;   // data output
  wire  [ 7:0] dat_o_pwm;   // data input
  wire         ack_o_pwm;         // normal bus termination
  wire         wb_pwm_sel;
  */   
  wire         cyc_i_pwm0;         // cycle
  wire         stb_i_pwm0;         // strobe
  wire         we_i_pwm0;          // write enable
  wire  [15:0] adr_i_pwm0;         // address adr_i
  wire  [15:0] dat_o_pwm0;   // data input
  wire         ack_o_pwm0;         // normal bus termination
  wire         wb_pwm0_sel;
  wire         cyc_i_pwm1;         // cycle
  wire         stb_i_pwm1;         // strobe
  wire  [15:0] adr_i_pwm1;         // address adr_i
  wire         we_i_pwm1;          // write enable
  wire  [15:0] dat_o_pwm1;   // data input
  wire         ack_o_pwm1;         // normal bus termination
  wire         wb_pwm1_sel;
                         
  //bram
  wire         cyc_i_bram;         // cycle
  wire         stb_i_bram;         // strobe
  wire         we_i_bram;          // write enable
  wire  [31:0] dat_o_bram;   // data input
  wire         ack_o_bram;         // normal bus termination
  wire         err_o_bram;         // normal bus termination
  wire         wb_bram_sel;
  reg         wb_bram_sel_mon;
  wire         inta_bram;
   
  //spi
  wire         cyc_i_spi;         // cycle
  wire         stb_i_spi;         // strobe
  wire         we_i_spi;          // write enable
  wire  [2:0] adr_i_spi;         // address adr_i
  wire  [7:0] dat_o_spi;   // data input
  wire         ack_o_spi;         // normal bus termination
  wire         wb_spi_sel;
  wire         inta_spi;
  
  //opencores PWM signals
  wire  i_extclk;
  wire  [15:0]i_DC;
  wire  i_valid_DC;  
  wire  o_pwm0;
  wire  o_pwm1;
   
  //opencores SPI signals
  wire spiClkOut;
  wire spiDataIn;
  wire spiDataOut;
  wire spiCS_n;
      
  //mon
  reg mon_mosi ;
  reg mon_miso ;
  reg mon_sclk ;
  reg mon_ss ;
  reg mon ;
   
  //reset 
  assign reset = ~resetb;
  assign rstn = resetb;
  assign reset_locked =((resetb==1'b0)||(locked==1'b0))?1'b1:1'b0;
  assign resetb_locked = ~reset_locked ;
  assign phy_rstb  = ~reset_locked ;  
  //gpio                  
  assign gpi = gpo;
   
  //led
  assign led = {2'b00,gpo[3:0],2'b00,o_pwm1,o_pwm0};
   
   
clk_gen clk_gen(
   .areset(reset),   
   .inclk0(clk_50m),
   .c0(clk_50m_i),   // 50MHz
   .c1(clk_100m_i),  //100MHz
   .c2(clk_20m_i),  //20MHz
   .locked(locked));
   
nios2e nios2e(
   .reset_reset_n(resetb_locked),
   .clk_clk(clk_100m_i),
   .uart_0_txd(UART_Tx),    
   .uart_0_rxd(UART_Rx),
   .gpo_export(gpo),
   .gpi_export(gpi),
  //wishbone nios2 -> IP
   .wb_CLK_O(WB_CLK),         // clock
   .wb_RST_O(WB_RST),         // reset 
   .wb_CYC_O(WB_CYC),         // cycle
   .wb_STB_O(WB_STB),         // strobe
   .wb_ADR_O(WB_ADR),         // address adr
   .wb_SEL_O(WB_SEL),         // byte_sel
   .wb_WE_O( WB_WE),          // write enable
   .wb_DAT_O(WB_DAT_I),       // data output
   .wb_DAT_I(WB_DAT_O),       // data input
   .wb_ACK_I(WB_ACK),         // normal bus termination
   .wb_ERR_I(WB_ERR),         // error bus termination
   .wb_RTY_I(WB_RTY)          // retry
        );

     
//opencores simple_gpio    
 PWM PWM_0(
  .i_wb_clk(WB_CLK),         // clock
  .i_wb_rst(WB_RST),        // reset (asynchronous active hi)
  .i_wb_cyc(cyc_i_pwm0),    // cycle
  .i_wb_stb(stb_i_pwm0),    // strobe
  .i_wb_adr(adr_i_pwm0),         // address adr_i[1]
  .i_wb_we(we_i_pwm0),      // write enable
  .i_wb_data(WB_DAT_I[15:0]),     // data output
  .o_wb_data(dat_o_pwm0),    // data input
  .o_wb_ack(ack_o_pwm0),    // normal bus termination
  .i_extclk(i_extclk),
  .i_DC(i_DC),
  .i_valid_DC(i_valid_DC),
  .o_pwm(o_pwm0)
                         ); 
 PWM PWM_1(
  .i_wb_clk(WB_CLK),       // clock
  .i_wb_rst(WB_RST),       // reset (asynchronous active hi)
  .i_wb_cyc(cyc_i_pwm1),    // cycle
  .i_wb_stb(stb_i_pwm1),    // strobe
  .i_wb_adr(adr_i_pwm1), // address adr_i
  .i_wb_we(we_i_pwm1),      // write enable
  .i_wb_data(WB_DAT_I[15:0]),     // data output
  .o_wb_data(dat_o_pwm1),   // data input
  .o_wb_ack(ack_o_pwm1),    // normal bus termination
  .i_extclk(i_extclk),
  .i_DC(i_DC),
  .i_valid_DC(i_valid_DC),
  .o_pwm(o_pwm1)
                         );

 //SPI Master
 spiMaster u_spiMaster (
  //Wishbone bus
  .clk_i(WB_CLK),
  .rst_i(WB_RST),
  .address_i(WB_ADR[7:0]),
  .data_i(WB_DAT_I[7:0]),
  .data_o(dat_o_spi),
  .strobe_i(stb_i_spi),
  .we_i(we_i_spi),
  .ack_o(ack_o_spi),

  // SPI logic clock
  .spiSysClk(clk_20m_i),

  //SPI bus
  .spiClkOut(spiClkOut),
  .spiDataIn(spiDataIn),
  .spiDataOut(spiDataOut),
  .spiCS_n(spiCS_n)
);

//WihsBoneBus interconnect
   assign wb_pwm0_sel = ((WB_ADR>=p_wb_pwm0_low)&&(WB_ADR<=p_wb_pwm0_hi))? 1'b1:1'b0;
   assign cyc_i_pwm0 =(wb_pwm0_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_pwm0 =(wb_pwm0_sel==1'b1)?WB_STB:1'b0;
   assign adr_i_pwm0 ={12'h000,WB_ADR[3:0]};
   assign we_i_pwm0  =(wb_pwm0_sel==1'b1)?WB_WE:1'b0;
   
   assign wb_pwm1_sel = ((WB_ADR>=p_wb_pwm1_low)&&(WB_ADR<=p_wb_pwm1_hi))? 1'b1:1'b0;
   assign cyc_i_pwm1 =(wb_pwm1_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_pwm1 =(wb_pwm1_sel==1'b1)?WB_STB:1'b0;
   assign adr_i_pwm1 ={12'h000,WB_ADR[3:0]};
   assign we_i_pwm1  =(wb_pwm1_sel==1'b1)?WB_WE:1'b0;

   assign wb_spi_sel = ((WB_ADR>=p_wb_spi_low)&&(WB_ADR<=p_wb_spi_hi))? 1'b1:1'b0;
   assign cyc_i_spi =(wb_spi_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_spi =(wb_spi_sel==1'b1)?WB_STB:1'b0;
   assign adr_i_spi =WB_ADR[2:0];
   assign we_i_spi  =(wb_spi_sel==1'b1)?WB_WE:1'b0;
                                    
   assign wb_bram_sel = ((WB_ADR>=p_wb_bram_low)&&(WB_ADR<=p_wb_bram_hi))? 1'b1:1'b0;
   assign cyc_i_bram =(wb_bram_sel==1'b1)?WB_CYC:1'b0;
   assign stb_i_bram =(wb_bram_sel==1'b1)?WB_STB:1'b0;
   assign we_i_bram  =(wb_bram_sel==1'b1)?WB_WE:1'b0;
   
                
   assign WB_DAT_O =(wb_pwm0_sel==1'b1)?{dat_o_pwm0,dat_o_pwm0}:
                    (wb_pwm1_sel==1'b1)?{dat_o_pwm1,dat_o_pwm1}:
                    (wb_spi_sel==1'b1)?{dat_o_spi,dat_o_spi,dat_o_spi,dat_o_spi}:
                    (wb_bram_sel==1'b1)?dat_o_bram:
                                                                                    32'h00000000;

   assign WB_ACK = (WB_ACK_DFF==1'b1)?1'b1:
                   (wb_pwm0_sel==1'b1)?ack_o_pwm0:
                   (wb_pwm1_sel==1'b1)?ack_o_pwm1:
                   (wb_spi_sel==1'b1)?ack_o_spi:
                   (wb_bram_sel==1'b1)?ack_o_bram:
                                        WB_STB;
   
always @ (posedge clk_100m_i or negedge resetb )
  begin
     if(resetb==1'b0)
         WB_ACK_DFF <= 1'b0;
     else  
       if (((wb_pwm0_sel==1'b1)&&(ack_o_pwm0==1'b1))||
           ((wb_pwm1_sel==1'b1)&&(ack_o_pwm1==1'b1))||
           ((wb_spi_sel==1'b1)&&(ack_o_spi==1'b1)))
         WB_ACK_DFF <= 1'b1;
       else
         if (WB_STB==1'b0)
           WB_ACK_DFF <= 1'b0;
         else
           WB_ACK_DFF <= WB_ACK_DFF ;
  end

   assign WB_ERR = 1'b0;
   assign WB_RTY = 1'b0;
    
// PWM inputs
   assign i_extclk =1'b0;
   assign i_DC = 16'h0000;
   assign i_valid_DC =1'b0;    
   
   
always@(posedge clk_20m_i )
  mon <= mon_mosi ^ mon_miso ^ mon_sclk ^ mon_ss;
   
always @ (posedge clk_20m_i  )
    begin
      mon_mosi <= mosi;
      mon_miso <= miso;
      mon_sclk <= sclk;
      mon_ss   <= ss;
    end
/*   
always @ (posedge clk_2m_i  )
    begin
    end
   
always @ (posedge clk_2m_i  )
    begin
    end
*/
    
wb_ram_if wb_ram_if_a
(
  // WISHBONE common
  .wb_clk_i(WB_CLK),
  .wb_rst_i(WB_RST),
  .wb_dat_i(WB_DAT_I),
  .wb_dat_o(dat_o_bram), 
  // WISHBONE slave
  .wb_adr_i(WB_ADR[12:2]),
  .wb_sel_i(WB_SEL),
  .wb_we_i(we_i_bram),
  .wb_cyc_i(cyc_i_bram), 
  .wb_stb_i(stb_i_bram),
  .wb_ack_o(ack_o_bram),
  .wb_err_o(err_o_bram), 
  //BRAM Port-A                
  .clkb(clka), // input clkb
  .web(wea), // input [3 : 0] web
  .addrb(addra), // input [8 : 0] addrb
  .dinb(dina), // input [31 : 0] dinb
  .doutb(douta) // output [31 : 0] doutb
 
);

   
assign wea_s = (wea==4'b0000)?1'b0:1'b1;
   
assign web_s = (web==4'b0000)?1'b0:1'b1;
   

    
ram_32b_512w_sp ram_32b_512w_u0 (
  .clock(clka), // input clka
  .byteena(wea),
  .wren(wea_s), // input [0 : 0] wea
  .address(addra), // input [8 : 0] addra
  .data(dina), // input [31 : 0] dina
  .q(douta) // output [31 : 0] doutb
);  

//SPI signal
   assign sclk = spiClkOut;
   assign spiDataIn = miso;
   assign mosi = spiDataOut;
   assign ss =  spiCS_n;

endmodule




